@@ -19,7 +19,7 @@ package Image::ExifTool::Shortcuts;
 use strict;
 use vars qw($VERSION);
 
-$VERSION = '1.60';
+$VERSION = '1.63';
 
 # this is a special table used to define command-line shortcuts
 # (documentation Notes may be added for these via %shortcutNotes in BuildTagLookup.pm)
@@ -167,6 +167,7 @@ $VERSION = '1.60';
         'MakerNoteLeica7',
         'MakerNoteLeica8',
         'MakerNoteLeica9',
+        'MakerNoteLeica10',
         'MakerNotePanasonic',
         'MakerNotePanasonic2',
         'MakerNotePanasonic3',
@@ -179,8 +180,10 @@ $VERSION = '1.60';
         'MakerNotePhaseOne',
         'MakerNoteReconyx',
         'MakerNoteReconyx2',
+        'MakerNoteReconyx3',
         'MakerNoteRicoh',
         'MakerNoteRicoh2',
+        'MakerNoteRicohPentax',
         'MakerNoteRicohText',
         'MakerNoteSamsung1a',
         'MakerNoteSamsung1b',
@@ -329,7 +332,7 @@ name prefix.
 
 =head1 AUTHOR
 
-Copyright 2003-2018, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2019, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
